/*******************************************************************************
  MPLAB Harmony Application Source File

  Company:
    Microchip Technology Inc.

  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It
    implements the logic of the application's state machine and it may call
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

/*******************************************************************************
* Copyright (C) 2019 Microchip Technology Inc. and its subsidiaries.
*
* Subject to your compliance with these terms, you may use Microchip software
* and any derivatives exclusively with Microchip products. It is your
* responsibility to comply with third party license terms applicable to your
* use of third party software (including open source software) that may
* accompany Microchip software.
*
* THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
* EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED
* WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
* PARTICULAR PURPOSE.
*
* IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
* INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
* WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
* BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE
* FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN
* ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY,
* THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
*******************************************************************************/
// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include "app.h"
#include "cryptoauthlib.h"
#include "host/atca_host.h"
#include "../../pubkey_rotation.h"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************
extern ATCAIfaceCfg atecc608_0_init_data;
extern uint8_t invalidated_nonce[];
extern uint8_t invalidated_signature[];
extern uint8_t public_key[];
extern uint8_t validated_nonce[];
extern uint8_t validated_signature[];
extern uint8_t rotating_digest[];
extern uint8_t rotating_signature[];
extern uint16_t rotating_pubkey_slot;
extern uint16_t authority_pubkey_slot;

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.

    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;


// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************
ATCA_STATUS authorize_rotating_public_key(uint8_t* nonce, uint8_t* genkey_other_data,
                                          uint8_t* verify_other_data, bool pubkey_state)
{
    ATCA_STATUS status = ATCA_GEN_FAIL;
    uint8_t rand_out[32];
    atca_temp_key_t temp_key;
    atca_nonce_in_out_t nonce_params;
    uint8_t ser_num[ATCA_SERIAL_NUM_SIZE];
    uint8_t dev_public_key[ATCA_PUB_KEY_SIZE];
    atca_gen_key_in_out_t gen_key_params;
    uint8_t config_data[ATCA_ECC_CONFIG_SIZE];
    uint8_t validation_msg[55];
    uint8_t validation_digest[ATCA_SHA256_DIGEST_SIZE];
    atca_sign_internal_in_out_t sign_params;

    // clear the buffer
    memset(&temp_key, 0, sizeof(temp_key));
    memset(&nonce_params, 0, sizeof(nonce_params));
    memset(&gen_key_params, 0, sizeof(gen_key_params));
    memset(&sign_params, 0, sizeof(sign_params));

    do
    {
        // set nonce parameters & Calculate nonce on host
        nonce_params.mode = NONCE_MODE_PASSTHROUGH;
        nonce_params.zero = 0;
        nonce_params.num_in = nonce;
        nonce_params.rand_out = rand_out;
        nonce_params.temp_key = &temp_key;
        if ((status = atcah_nonce(&nonce_params)) != ATCA_SUCCESS)
        {
            printf("Nonce calculation on host is failed{%02X}.\r\n", status);
            break;
        }

        if ((status = atcab_read_serial_number(ser_num)) != ATCA_SUCCESS)
        {
            printf("Reading serial number is failed{%02X}.\r\n", status);
            break;
        }

        //This configuration allows reading public key, if not, host should
        // maintain the track of public key in the slot
        //Read the public key from the rotating public key slot
        if ((status = atcab_read_pubkey(rotating_pubkey_slot, dev_public_key)) != ATCA_SUCCESS)
        {
            printf("Reading public key from device is failed{%02X}.\r\n", status);
            break;
        }

        // set genkey params & Calculate rotating public key digest
        gen_key_params.mode = GENKEY_MODE_PUBKEY_DIGEST;
        gen_key_params.key_id = rotating_pubkey_slot;
        gen_key_params.public_key = dev_public_key;
        gen_key_params.public_key_size = sizeof(dev_public_key);
        gen_key_params.other_data = genkey_other_data;
        gen_key_params.sn = ser_num;
        gen_key_params.temp_key = &temp_key;
        if ((status = atcah_gen_key_msg(&gen_key_params)) != ATCA_SUCCESS)
        {
            printf("Calculating pubkey digest on host is failed{%02X}.\r\n", status);
            break;
        }

        if ((status = atcab_read_config_zone(config_data)) != ATCA_SUCCESS)
        {
            printf("Reading configuration zone is failed{%02X}.\r\n", status);
            break;
        }

        // set sign internal params & Calculate sign internal message
        sign_params.sn = ser_num;
        sign_params.verify_other_data = verify_other_data;
        sign_params.key_id = authority_pubkey_slot;
        sign_params.slot_config = (config_data[20 + rotating_pubkey_slot * 2]   | config_data[21 + rotating_pubkey_slot * 2] << 8);
        sign_params.key_config = (config_data[96 + rotating_pubkey_slot * 2] | config_data[97 + rotating_pubkey_slot * 2] << 8);
        sign_params.for_invalidate = !pubkey_state;
        sign_params.message = validation_msg;
        sign_params.digest = validation_digest;
        sign_params.temp_key = &temp_key;
        if ((status = atcah_sign_internal_msg(ATECC608A, &sign_params)) != ATCA_SUCCESS)
        {
            printf("Calculating sign internal message on host is failed{%02X}.\r\n", status);
            break;
        }

    }
    while (0);

    return status;
}

ATCA_STATUS invalidate_existing_pubkey(void)
{
    ATCA_STATUS status = ATCA_GEN_FAIL;
    uint8_t valid_bytes[4];
    uint8_t nonce[32];
    uint8_t genkey_other_data[3];
    uint8_t verify_other_data[19];
    bool is_verified = false;

    do
    {
        // check public key slot is in valid or invalid state
        if ((status = atcab_read_zone(ATCA_ZONE_DATA, rotating_pubkey_slot, 0, 0, valid_bytes,
                                      sizeof(valid_bytes))) != ATCA_SUCCESS)
        {
            printf("Reading public key valid bytes is failed{%02X}.\r\n", status);
            break;
        }

        // 0x0A - Invalid state
        // 0x05 - Valid state
        if ((valid_bytes[0] >> 4) == 0x0A)
        {
            printf("Existing public key in the slot is already invalidated.\r\n");
            status = ATCA_SUCCESS;
            break;
        }

        // Initialize the buffer
        memcpy(nonce, invalidated_nonce, sizeof(nonce));
        memset(genkey_other_data, 0, sizeof(genkey_other_data));
        memset(verify_other_data, 0, sizeof(verify_other_data));

        if ((status = authorize_rotating_public_key(nonce, genkey_other_data, verify_other_data,
                                                    false)) != ATCA_SUCCESS)
        {
            printf("Authorizing existing public key is failed{%02X}.\r\n", status);
            break;
        }

        if ((status = atcab_nonce(nonce)) != ATCA_SUCCESS)
        {
            printf("Load nonce into tempkey is failed{%02X}.\r\n", status);
            break;
        }

        if ((status = atcab_genkey_base(GENKEY_MODE_PUBKEY_DIGEST, rotating_pubkey_slot, genkey_other_data,
                                        NULL)) != ATCA_SUCCESS)
        {
            printf("Generate pubkey digest on device is failed{%02X}.\r\n", status);
            break;
        }

        if ((status = atcab_verify_invalidate(rotating_pubkey_slot, invalidated_signature, verify_other_data,
                                              &is_verified)) != ATCA_SUCCESS)
        {
            printf("Verify invalidate is failed{%02X}.\r\n", status);
            break;
        }

        if (is_verified)
        {
            printf("Existing public key invalidation is successful.\r\n");
        }
        else
        {
            status = ATCA_GEN_FAIL;
            printf("Existing public key invalidation is failed{%02X}.\r\n", status);
        }
    }
    while (0);

    return status;
}

ATCA_STATUS write_new_pubkey(void)
{
    ATCA_STATUS status = ATCA_GEN_FAIL;
    char displaystr[400];
    size_t displaylen;

    status = atcab_write_pubkey(rotating_pubkey_slot, public_key);

    if (status == ATCA_SUCCESS)
    {
        displaylen = sizeof(displaystr);
        atcab_bin2hex(public_key, ATCA_PUB_KEY_SIZE, displaystr, &displaylen);
        printf("New Public key is written to device:\r\n\r\n%s\r\n\r\n", displaystr);
    }
    else
    {
        printf("Writing new public key to slot is failed{%02X}.\r\n", status);
    }

    return status;
}

ATCA_STATUS validate_new_pubkey(void)
{
    ATCA_STATUS status = ATCA_GEN_FAIL;
    uint8_t nonce[32];
    uint8_t genkey_other_data[3];
    uint8_t verify_other_data[19];
    bool is_verified = false;

    do
    {
        // Initialize the buffer
        memcpy(nonce, validated_nonce, sizeof(nonce));
        memset(genkey_other_data, 0, sizeof(genkey_other_data));
        memset(verify_other_data, 0, sizeof(verify_other_data));

        if ((status = authorize_rotating_public_key(nonce, genkey_other_data, verify_other_data,
                                                    true)) != ATCA_SUCCESS)
        {
            printf("Authorizing new public key is failed{%02X}.\r\n", status);
            break;
        }

        if ((status = atcab_nonce(nonce)) != ATCA_SUCCESS)
        {
            printf("Load nonce into tempkey is failed{%02X}.\r\n", status);
            break;
        }

        if ((status = atcab_genkey_base(GENKEY_MODE_PUBKEY_DIGEST, rotating_pubkey_slot, genkey_other_data,
                                        NULL)) != ATCA_SUCCESS)
        {
            printf("Generate pubkey digest on device is failed{%02X}.\r\n", status);
            break;
        }

        if ((status = atcab_verify_validate(rotating_pubkey_slot, validated_signature, verify_other_data,
                                            &is_verified)) != ATCA_SUCCESS)
        {
            printf("Verify validate is failed{%02X}.\r\n", status);
            break;
        }

        if (is_verified)
        {
            printf("New public key validatation is successful.\r\n");
        }
        else
        {
            status = ATCA_GEN_FAIL;
            printf("New public key validation is failed{%02X}.\r\n", status);
        }

    }
    while (0);

    return status;
}

ATCA_STATUS verify_new_pubkey(void)
{
    ATCA_STATUS status = ATCA_GEN_FAIL;
    bool is_verified = false;

    status = atcab_verify_stored(rotating_digest, rotating_signature, rotating_pubkey_slot,
                                 &is_verified);
    if (status != ATCA_SUCCESS)
    {
        is_verified = false;
        printf("atcab_verify_stored failed{%02X}.\r\n", status);
    }

    if (is_verified)
    {
        printf("New public key is able to perform verify operations successfully.\r\n");
    }
    else
    {
        status = ATCA_GEN_FAIL;
        printf("New public key is unable to perform verify operations{%02X}.\r\n", status);
    }

    return status;
}

// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */
void APP_Initialize(void)
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;
}

/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */
void APP_Tasks(void)
{
    ATCA_STATUS status;

    switch(appData.state)
    {
        case APP_STATE_INIT:
            status = atcab_init(&atecc608_0_init_data);
            appData.state = (status == ATCA_SUCCESS) ? APP_STATE_INVALIDATE_PUBKEY : APP_STATE_FINISH;
            break;

        case APP_STATE_INVALIDATE_PUBKEY:
            status = invalidate_existing_pubkey();
            appData.state = (status == ATCA_SUCCESS) ? APP_STATE_UPDATE_AND_VALIDATE_PUBKEY : APP_STATE_FINISH;
            break;

        case APP_STATE_UPDATE_AND_VALIDATE_PUBKEY:
            status = write_new_pubkey();
            if (status == ATCA_SUCCESS)
            {
                status = validate_new_pubkey();
            }
            appData.state = (status == ATCA_SUCCESS) ? APP_STATE_VERIFY_PUBKEY : APP_STATE_FINISH;
            break;

        case APP_STATE_VERIFY_PUBKEY:
            (void)verify_new_pubkey();
            appData.state = APP_STATE_FINISH;
            break;

        case APP_STATE_FINISH:
        /* Intentional fall through */
        default:
            /* Nothing to do */
            break;
    }
}

/*******************************************************************************
 End of File
 */
